@@ -13,23 +13,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-LVM class for performing LVM operations.
-"""
+"""LVM class for performing LVM operations."""
 
 import math
 import os
 import re
 
+from oslo_concurrency import processutils as putils
+from oslo_log import log as logging
+from oslo_utils import excutils
+
 from os_brick import exception
 from os_brick import executor
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick import utils
-from oslo_concurrency import processutils as putils
-from oslo_log import log as logging
-from oslo_utils import excutils
-from six import moves
-
 
 LOG = logging.getLogger(__name__)
 
@@ -41,7 +38,8 @@ class LVM(executor.Executor):
 
     def __init__(self, vg_name, root_helper, create_vg=False,
                  physical_volumes=None, lvm_type='default',
-                 executor=None, lvm_conf=None):
+                 executor=None, lvm_conf=None,
+                 suppress_fd_warn=False):
 
         """Initialize the LVM object.
 
@@ -56,7 +54,7 @@ class LVM(executor.Executor):
         :param lvm_type: VG and Volume type (default, or thin)
         :param executor: Execute method to use, None uses
                          oslo_concurrency.processutils
-
+        :param suppress_fd_warn: Add suppress FD Warn to LVM env
         """
         super(LVM, self).__init__(execute=executor, root_helper=root_helper)
         self.vg_name = vg_name
@@ -75,11 +73,19 @@ class LVM(executor.Executor):
         # Ensure LVM_SYSTEM_DIR has been added to LVM.LVM_CMD_PREFIX
         # before the first LVM command is executed, and use the directory
         # where the specified lvm_conf file is located as the value.
+
+        # NOTE(jdg): We use the temp var here becuase LVM_CMD_PREFIX is a
+        # class global and if you use append here, you'll literally just keep
+        # appending values to the global.
+        _lvm_cmd_prefix = ['env', 'LC_ALL=C']
+
         if lvm_conf and os.path.isfile(lvm_conf):
             lvm_sys_dir = os.path.dirname(lvm_conf)
-            LVM.LVM_CMD_PREFIX = ['env',
-                                  'LC_ALL=C',
-                                  'LVM_SYSTEM_DIR=' + lvm_sys_dir]
+            _lvm_cmd_prefix.append('LVM_SYSTEM_DIR=' + lvm_sys_dir)
+
+        if suppress_fd_warn:
+            _lvm_cmd_prefix.append('LVM_SUPPRESS_FD_WARNINGS=1')
+        LVM.LVM_CMD_PREFIX = _lvm_cmd_prefix
 
         if create_vg and physical_volumes is not None:
             try:
@@ -177,7 +183,10 @@ class LVM(executor.Executor):
                 consumed_space = pool_size / 100 * data_percent
                 free_space = pool_size - consumed_space
                 free_space = round(free_space, 2)
-        except putils.ProcessExecutionError as err:
+        # Need noqa due to a false error about the 'err' variable being unused
+        # even though it is used in the logging. Possibly related to
+        # https://github.com/PyCQA/pyflakes/issues/378.
+        except putils.ProcessExecutionError as err:  # noqa
             LOG.exception('Error querying thin pool about data_percent')
             LOG.error('Cmd     :%s', err.cmd)
             LOG.error('StdOut  :%s', err.stdout)
@@ -303,7 +312,7 @@ class LVM(executor.Executor):
         lv_list = []
         if out is not None:
             volumes = out.split()
-            iterator = moves.zip(*[iter(volumes)] * 3)  # pylint: disable=E1101
+            iterator = zip(*[iter(volumes)] * 3)  # pylint: disable=E1101
             for vg, name, size in iterator:
                 lv_list.append({"vg": vg, "name": name, "size": size})
 
